"use strict";(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[551],{94551:function(e,t,r){r.r(t),r.d(t,{default:function(){return l}});r(47313);var i=r(69367),s=r(50412),n=r(46417);var l=function(e){var t=e.disabled,r=e.element,l=e.widgetMgr,u={width:e.width},d="primary"===r.type?i.hY.PRIMARY:i.hY.SECONDARY;return(0,n.jsx)("div",{className:"row-widget stButton",style:u,children:(0,n.jsx)(i.uD,{help:r.help,children:(0,n.jsx)(i.ZP,{kind:d,size:i.$u.SMALL,disabled:t,fluidWidth:r.useContainerWidth||!1,onClick:function(){return l.setTriggerValue(r,{fromUi:!0})},children:(0,n.jsx)(s.Z,{source:r.label,allowHTML:!1,isButton:!0})})})})}}}]);