"use strict";(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[341],{7341:function(t,e,r){r.r(e),r.d(e,{default:function(){return a}});var i=r(47313),n=r(26419),o=r(46417);var a=(0,n.Z)((function(t){var e=t.width,r=t.element,n=t.height,a="bokeh-chart-".concat(r.elementId),c=(0,i.useCallback)((function(){return JSON.parse(r.figure)}),[r]),u=(0,i.useCallback)((function(t){var i=window.Bokeh,o=document.getElementById(a),c=t&&t.doc&&t.doc.roots&&t.doc.roots.references?t.doc.roots.references.find((function(t){return"Plot"===t.type})):void 0;if(c){var u=function(t){var i=t.attributes.plot_width,o=t.attributes.plot_height;return n?(i=e,o=n):r.useContainerWidth&&(i=e),{chartWidth:i,chartHeight:o}}(c),s=u.chartWidth,h=u.chartHeight;s>0&&(c.attributes.plot_width=s),h>0&&(c.attributes.plot_height=h)}null!==o&&(!function(t){for(;t.lastChild;)t.lastChild.remove()}(o),i.embed.embed_item(t,a))}),[e,n,r]);return(0,i.useEffect)((function(){u(c())}),[e,n,r,c,u]),(0,o.jsx)("div",{id:a,className:"stBokehChart"})}))}}]);