"use strict";(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[807],{99807:function(n,r,t){t.r(r),t.d(r,{default:function(){return c}});t(47313);var o=t(4371),e=t(47167),i=t(48069),a=(0,e.Z)("div",{target:"edb2rvg0"})((function(n){var r=n.theme;return{paddingBottom:r.spacing.smPx,lineHeight:r.lineHeights.normal,color:(0,i.yq)(r)}}),""),s=t(50412),g=t(46417);var c=function(n){var r=n.element,t=n.width;return(0,g.jsxs)("div",{className:"stProgress",children:[(0,g.jsx)(a,{children:(0,g.jsx)(s.Z,{source:r.text,allowHTML:!1,isLabel:!0})}),(0,g.jsx)(o.Z,{value:r.value,width:t})]})}},4371:function(n,r,t){t.d(r,{$:function(){return o},Z:function(){return d}});var o,e=t(47313),i=t(83985),a=t(49950),s=t(48069),g=t(75314),c=t(74969),u=t(46417);!function(n){n.EXTRASMALL="xs",n.SMALL="sm",n.MEDIUM="md",n.LARGE="lg",n.EXTRALARGE="xl"}(o||(o={}));var d=function(n){var r=n.value,t=n.width,d=n.size,l=void 0===d?o.SMALL:d,p=n.overrides,m=(0,i.u)(),h={xs:m.spacing.twoXS,sm:m.spacing.sm,md:m.spacing.lg,lg:m.spacing.xl,xl:m.spacing.twoXL},f=e.useContext(a.Z).activeTheme,v=!(0,s.MJ)(f),w={BarContainer:{style:{marginTop:m.spacing.none,marginBottom:m.spacing.none,marginRight:m.spacing.none,marginLeft:m.spacing.none}},Bar:{style:function(n){var r=n.$theme;return{width:t?t.toString():void 0,marginTop:m.spacing.none,marginBottom:m.spacing.none,marginRight:m.spacing.none,marginLeft:m.spacing.none,height:h[l],backgroundColor:r.colors.progressbarTrackFill,borderTopLeftRadius:m.spacing.twoXS,borderTopRightRadius:m.spacing.twoXS,borderBottomLeftRadius:m.spacing.twoXS,borderBottomRightRadius:m.spacing.twoXS}}},BarProgress:{style:function(n){n.$theme;return{backgroundColor:v?m.colors.primary:m.colors.blue70,borderTopLeftRadius:m.spacing.twoXS,borderTopRightRadius:m.spacing.twoXS,borderBottomLeftRadius:m.spacing.twoXS,borderBottomRightRadius:m.spacing.twoXS}}}};return(0,u.jsx)(g.Z,{value:r,overrides:(0,c.aO)(w,p)})}}}]);