"use strict";(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[830],{10830:function(e,t,r){r.r(t),r.d(t,{default:function(){return o}});var i=r(47313),n=r(49950),l=r(69367),s=r(50412),a=r(93986),u=r(46417);var o=function(e){var t=e.disabled,r=e.element,o=e.widgetMgr,c={width:e.width},d=(0,i.useContext)(n.Z).getBaseUriParts;return(0,u.jsx)("div",{className:"row-widget stDownloadButton",style:c,children:(0,u.jsx)(l.uD,{help:r.help,children:(0,u.jsx)(l.ZP,{kind:l.hY.SECONDARY,size:l.$u.SMALL,disabled:t,onClick:function(){o.setTriggerValue(r,{fromUi:!0});var e=document.createElement("a"),t="".concat((0,a.Bz)(r.url,d()),"?title=").concat(encodeURIComponent(document.title));e.setAttribute("href",t),e.setAttribute("target","_blank"),e.click()},fluidWidth:r.useContainerWidth||!1,children:(0,u.jsx)(s.Z,{source:r.label,allowHTML:!1,isButton:!0})})})})}}}]);