"use strict";(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[248],{80248:function(e,t,r){r.r(t),r.d(t,{default:function(){return a}});var s=r(47313),u=r(49950),n=r(93986),i=r(46417);function a(e){var t=e.element,r=e.width,a=(0,s.useRef)(null),c=(0,s.useContext)(u.Z).getBaseUriParts;(0,s.useEffect)((function(){a.current&&(a.current.currentTime=t.startTime)}),[t.startTime]);var l=(0,n.Bz)(t.url,c());return(0,i.jsx)("audio",{id:"audio",ref:a,controls:!0,src:l,className:"stAudio",style:{width:r}})}}}]);