(self.webpackChunkstreamlit_browser=self.webpackChunkstreamlit_browser||[]).push([[705],{9441:function(t,i,e){"use strict";e.r(i),e.d(i,{default:function(){return m}});var r,n=e(47313),a=e(16390),o=e(49950),d=e(26419),l=e(93986),s=e(47167),c=(0,s.Z)("div",{target:"etr89bj2"})((function(t){return{display:"flex",flexDirection:"row",flexWrap:"wrap",rowGap:t.theme.spacing.lg}}),""),u=(0,s.Z)("div",{target:"etr89bj1"})((function(t){t.theme;return{display:"flex",flexDirection:"column",alignItems:"stretch",width:"auto",flexGrow:0}}),""),h=(0,s.Z)("div",{target:"etr89bj0"})((function(t){var i=t.theme;return{fontFamily:i.genericFonts.bodyFont,fontSize:i.fontSizes.sm,color:i.colors.fadedText60,textAlign:"center",marginTop:i.spacing.xs,wordWrap:"break-word",padding:"0.125rem"}}),""),f=e(46417);!function(t){t[t.OriginalWidth=-1]="OriginalWidth",t[t.ColumnWidth=-2]="ColumnWidth",t[t.AutoWidth=-3]="AutoWidth"}(r||(r={}));var m=(0,d.Z)((function(t){var i,e=t.width,d=t.isFullScreen,s=t.element,m=t.height,g=(0,n.useContext)(o.Z).getBaseUriParts,p=s.width;if(p===r.OriginalWidth||p===r.AutoWidth)i=void 0;else if(p===r.ColumnWidth)i=e;else{if(!(p>0))throw Error("Invalid image width: ".concat(p));i=p}var w={};return m&&d?(w.maxHeight=m,w["object-fit"]="contain"):(w.width=i,p===r.AutoWidth&&(w.maxWidth="100%")),(0,f.jsx)(c,{style:{width:e},children:s.imgs.map((function(t,i){var e=t;return(0,f.jsxs)(u,{"data-testid":"stImage",children:[e.markup?(0,a.ZP)((0,l.kB)(e.markup)):(0,f.jsx)("img",{style:w,src:(0,l.Bz)(e.url,g()),alt:i.toString()}),e.caption&&(0,f.jsx)(h,{"data-testid":"caption",style:w,children:" ".concat(e.caption," ")})]},i)}))})}))},50247:function(){}}]);